@@ -737,6 +737,13 @@ impl<T> GenerationalTokenList<T> {
     /// ```
     pub fn iter(&self) -> Iter<T> {
         Iter {
+            inner: self.iter_with_tokens(),
+        }
+    }
+
+    /// Returns an iterator of pairs of (item tokens, references to item data) in the list.
+    pub fn iter_with_tokens(&self) -> IterWithTokens<T> {
+        IterWithTokens {
             list: self,
             next_item: self.head,
         }
@@ -760,8 +767,16 @@ impl<T> GenerationalTokenList<T> {
     /// ```
     #[cfg(feature = "iter-mut")]
     pub fn iter_mut(&mut self) -> IterMut<T> {
-        let head = self.head;
         IterMut {
+            inner: self.iter_with_tokens_mut(),
+        }
+    }
+
+    /// Returns an iterator of pairs of (item tokens, mutable (exclusive) references to item data) in the list.
+    #[cfg(feature = "iter-mut")]
+    pub fn iter_with_tokens_mut(&mut self) -> IterWithTokensMut<T> {
+        let head = self.head;
+        IterWithTokensMut {
             list: self,
             next_item: head,
         }
@@ -811,7 +826,7 @@ impl<T> GenerationalTokenList<T> {
 }
 
 #[cfg(feature = "iter-mut")]
-pub struct IterMut<'a, T>
+pub struct IterWithTokensMut<'a, T>
 where
     T: 'a,
 {
@@ -819,6 +834,34 @@ where
     next_item: Option<ItemToken>,
 }
 
+#[cfg(feature = "iter-mut")]
+impl<'a, T> Iterator for IterWithTokensMut<'a, T>
+where
+    T: 'a,
+{
+    type Item = (ItemToken, &'a mut T);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let next_item = self.next_item?;
+
+        self.list.arena.get_mut(next_item.index).map(|i| {
+            self.next_item = i.next;
+
+            #[cfg_attr(feature = "iter-mut", allow(unsafe_code))]
+            let data = unsafe { &mut *(&mut i.data as *mut T) };
+            (next_item, data)
+        })
+    }
+}
+
+#[cfg(feature = "iter-mut")]
+pub struct IterMut<'a, T>
+where
+    T: 'a,
+{
+    inner: IterWithTokensMut<'a, T>,
+}
+
 #[cfg(feature = "iter-mut")]
 impl<'a, T> Iterator for IterMut<'a, T>
 where
@@ -826,14 +869,31 @@ where
 {
     type Item = &'a mut T;
 
+    fn next(&mut self) -> Option<Self::Item> {
+        self.inner.next().map(|d| d.1)
+    }
+}
+
+pub struct IterWithTokens<'a, T>
+where
+    T: 'a,
+{
+    list: &'a GenerationalTokenList<T>,
+    next_item: Option<ItemToken>,
+}
+
+impl<'a, T> Iterator for IterWithTokens<'a, T>
+where
+    T: 'a,
+{
+    type Item = (ItemToken, &'a T);
+
     fn next(&mut self) -> Option<Self::Item> {
         let next_item = self.next_item?;
 
-        self.list.arena.get_mut(next_item.index).map(|i| {
+        self.list.arena.get(next_item.index).map(|i| {
             self.next_item = i.next;
-
-            #[cfg_attr(feature = "iter-mut", allow(unsafe_code))]
-            unsafe { &mut *(&mut i.data as *mut T) }
+            (next_item, &i.data)
         })
     }
 }
@@ -842,8 +902,7 @@ pub struct Iter<'a, T>
 where
     T: 'a,
 {
-    list: &'a GenerationalTokenList<T>,
-    next_item: Option<ItemToken>,
+    inner: IterWithTokens<'a, T>,
 }
 
 impl<'a, T> Iterator for Iter<'a, T>
@@ -853,12 +912,7 @@ where
     type Item = &'a T;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let next_item = self.next_item?;
-
-        self.list.arena.get(next_item.index).map(|i| {
-            self.next_item = i.next;
-            &i.data
-        })
+        self.inner.next().map(|d| d.1)
     }
 }
 
@@ -945,6 +999,20 @@ where
     }
 }
 
+impl<T> std::ops::Index<ItemToken> for GenerationalTokenList<T> {
+    type Output = T;
+
+    fn index(&self, token: ItemToken) -> &Self::Output {
+        self.get(token).unwrap()
+    }
+}
+
+impl<T> std::ops::IndexMut<ItemToken> for GenerationalTokenList<T> {
+    fn index_mut(&mut self, token: ItemToken) -> &mut Self::Output {
+        self.get_mut(token).unwrap()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use pretty_assertions::assert_eq;
@@ -1272,4 +1340,66 @@ mod tests {
         let data = list.into_iter().collect::<Vec<_>>();
         assert_eq!(data, vec![10, 20, 30]);
     }
+
+    #[test]
+    fn index() {
+        let mut list = GenerationalTokenList::<i32>::new();
+        let item1 = list.push_back(10);
+        let item2 = list.push_back(20);
+        let item3 = list.push_back(30);
+        assert_eq!(list[item1], 10);
+        assert_eq!(list[item2], 20);
+        assert_eq!(list[item3], 30);
+    }
+
+    #[test]
+    fn index_mut() {
+        let mut list = GenerationalTokenList::<i32>::new();
+        let item1 = list.push_back(10);
+        let item2 = list.push_back(20);
+        let item3 = list.push_back(30);
+
+        list[item1] = 500;
+        list[item3] *= 9;
+
+        assert_eq!(list[item1], 500);
+        assert_eq!(list[item2], 20);
+        assert_eq!(list[item3], 270);
+    }
+
+    #[test]
+    fn iter_with_tokens() {
+        let mut list = GenerationalTokenList::<i32>::new();
+        let item1 = list.push_back(10);
+        let item2 = list.push_back(20);
+        let item3 = list.push_back(30);
+
+        let mut iter = list.iter_with_tokens();
+        assert_eq!(iter.next(), Some((item1, &10)));
+        assert_eq!(iter.next(), Some((item2, &20)));
+        assert_eq!(iter.next(), Some((item3, &30)));
+        assert_eq!(iter.next(), None);
+    }
+
+    #[test]
+    fn iter_with_tokens_mut() {
+        let mut list = GenerationalTokenList::<i32>::new();
+        let item1 = list.push_back(10);
+        let item2 = list.push_back(20);
+        let item3 = list.push_back(30);
+
+        let mut iter = list.iter_with_tokens_mut();
+        let i1 = iter.next().unwrap();
+        assert_eq!(i1.0, item1);
+        *i1.1 *= 2;
+        let i2 = iter.next().unwrap();
+        let i3 = iter.next().unwrap();
+        *i2.1 *= 3;
+        *i3.1 *= 4;
+        assert_eq!(i2.0, item2);
+        assert_eq!(i3.0, item3);
+
+        let data = list.into_iter().collect::<Vec<_>>();
+        assert_eq!(data, vec![20, 60, 120]);
+    }
 }
